'*******************************************************
' MM.DSO a simple Digital Storage Oscilloscope (kind of)
' this is a quick & dirty Version 0.10 beta
' for frequencies from 1 to 100 Hz (or more)
'
' System: MMBasic 4.7 (b32)
' Micromite 2 (MX170/28, Micks BP170 + 2.4" TFT)
' by twofingers 10-2015 at TBS
'
' Used keys (serial):
' TAB  switches gain (3V/30V) vertical res.
' I    enable/disable info text
' 1-4  Spread increase/decrease vertical
' +-   Waitstates increase/decrease
' ><   increase/decrease hor. grid
' " "  <Space> = freese
' q    quits
'
' Used PINs (MX170/28):
' 23 as Input
' 26 as Output (for Demosignals(
'*******************************************************
' This code may be freely distributed and changed.
' Provided AS IS without any warranty.
' Use it at your own risk. All information is provided for
' educational purposes only!
' ------------------------------------------------------
'  ++++++ Credit to Geoff for his great MMBasic ++++++
'*******************************************************



Option explicit
CPU 48
Const border_r=25                  ' voltage scala
Const screenwidth=MM.HRes-border_r ' usable screenwidth

Dim float p(screenwidth),y,y0,F,tm,w,x_base,vsum,VCorrect=1  'float is faster than int
Dim integer i, x=0 'x=hor_pos
Dim integer baseline, waitstates, gain=70,level=0,hGrid=10, Info=1,Freese=0


On KEY Keyhandler

Colour RGB(GREEN), RGB(BLACK)
CLS

SetPin 23,AIN   'Input

'************* Demo Signals  **************
'SetPin 26,DOUT  'Output  signal generator
'SetTick 20,SigGen
' or
'PWM 2,50,50
' or
'Servo 2,18
'******************************************

baseline=MM.VRes-10
F=1           'hor spread              default = 1
waitstates=0: 'slow sample speed down, default = 0

SetupScreen

y0=baseline


Do
  If x=0 Then
    If waitstates Then ' To be as fast as possible, we use 2 Sample Loops
       tm=Timer
       For i = 0 To screenwidth:p(i)=Pin(23):wait:Next
    Else
       tm=Timer
       For i = 0 To screenwidth:p(i)=Pin(23):Next
    EndIf
    tm=Timer-tm
    'Print tm:End
    x_base=screenwidth/tm  '4,6s/ms

    Box 0, 0,screenwidth, MM.VRes, 0,0,0   'CLS
    SetupScreen

    For i = Hgrid*F To tm Step Hgrid*F ' 10=1/100sec = 1000ms/10
       Line x_base*i, 0,x_base*i, MM.VRes, , RGB(blue)
    Next i
    vsum=0:For i = 1 To screenwidth:vsum=vsum+p(i) :Next
    y0=baseline-p(1/F)*gain
    'For i = 1 To screenwidth: Print p(i):Next
  EndIf

  x=x+1
  If (x+border_r)*F>=(MM.HRes*F) Then 'new screen
    Pause 500
    x=0
    Do:Loop While Freese
  EndIf
  y=baseline-p(x/F)*gain
  If x>1 Then Line x,y0,x,y
  y0=y

Loop


Sub SigGen 'settick interrupt
  If level = 0 Then level = 1 Else level = 0
  Pin(26)=level
End Sub


Sub SetupScreen
  Local scalefactor=70
  Local integer sc0=MM.VRes-baseline, sc1=sc0+scalefactor
  Local integer sc2=sc0+scalefactor*2,sc3=sc0+scalefactor*3
  Local string volt(4,1) = (""," 0V"," 1V"," 2V "," 3V",""," 0V","10V","20V","30V") LENGTH 3
  Local integer g=(gain=7)

  Line 0, MM.VRes-sc0,MM.HRes, MM.VRes-sc0, , RGB(red) '0V line
  Line 0, MM.VRes-sc1,MM.HRes, MM.VRes-sc1, , RGB(red) '
  Line 0, MM.VRes-sc2,MM.HRes, MM.VRes-sc2, , RGB(red) '
  Line 0, MM.VRes-sc3,MM.HRes, MM.VRes-sc3, , RGB(red) '3/30V line

  Text MM.HRes-border_r, MM.VRes-sc0-13, volt(1,g), , 2,1,RGB(YELLOW)
  Text MM.HRes-border_r, MM.VRes-sc1-13, volt(2,g), , 2,1,RGB(YELLOW)
  Text MM.HRes-border_r, MM.VRes-sc2-13, volt(3,g), , 2,1,RGB(YELLOW)
  Text MM.HRes-border_r, MM.VRes-sc3-13, volt(4,g), , 2,1,RGB(YELLOW)

  If Info Then
    Text 0,0,  "hGrid    "+Str$(Hgrid)+"ms", , 2,1,RGB(WHITE)
    Text 0,15, "Waitst   "+Str$(Waitstates), , 2,1,RGB(WHITE)
    Text 0,30, "Spread:  "+Str$(F), , 2,1,RGB(WHITE)
    Text 0,45, "avg.Volt"+Str$(vsum/screenwidth,2,2), , 2,1,RGB(WHITE)
  EndIf
End Sub


Sub wait
  For w=1 To waitstates:Next
End Sub


Sub keyhandler 'kbd interrupt handler
  On key 0
  Local key$
  key$=Inkey$
  Do While Inkey$=" ":Loop
  Select Case key$
   Case "1" To "4"
     F=Val(key$)
   Case "I","i"
     Info=Not Info
   Case "+"
     If Waitstates=0 Then
        Waitstates=1
     Else If Waitstates<10000 Then
        Waitstates=Waitstates*10
     EndIf
   Case "-"
     If Waitstates=1 Then
         Waitstates=0
     Else If Waitstates>0 Then
          Waitstates=Waitstates/10
     EndIf
   Case ">"
     If Hgrid<10000 Then Hgrid=Hgrid*10
   Case "<"
     If Hgrid>10 Then Hgrid=Hgrid/10
   Case Chr$(9)
     If gain = 70 Then
        gain=7
     Else
        gain=70
     EndIf
   Case "q"
     End
   Case " "
     Freese=Not Freese

  End Select
On key keyhandler
End Sub

End